/************************************************************************
 *
 * Copyright (C) 2020-2022 IRCAD France
 * Copyright (C) 2016 IHU Strasbourg
 *
 * This file is part of Sight.
 *
 * Sight is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Sight is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sight. If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include "MonoCamera.hpp"

#include <activity/validator/registry/macros.hpp>

#include <data/Camera.hpp>
#include <data/CameraSet.hpp>

namespace sight::module::activity::validator::CameraSet
{

SIGHT_REGISTER_ACTIVITY_VALIDATOR(sight::module::activity::validator::CameraSet::MonoCamera);

//-----------------------------------------------------------------------------

MonoCamera::MonoCamera(sight::activity::IValidator::Key /*unused*/)
{
}

//-----------------------------------------------------------------------------

sight::activity::IValidator::ValidationType MonoCamera::validate(const data::Object::csptr& currentData) const
{
    IValidator::ValidationType validation;

    validation.first  = true;
    validation.second = "";

    const auto& camera_set = data::CameraSet::dynamicConstCast(currentData);

    if(!camera_set)
    {
        validation.first  = false;
        validation.second = "Current data should be a CameraSet.";
    }
    else
    {
        if(camera_set->size() == 1)
        {
            data::Camera::csptr camera = camera_set->get_camera(0);
            if(!camera->getIsCalibrated())
            {
                validation.first  = false;
                validation.second = "The CameraSet should be calibrated.";
            }
        }
        else
        {
            validation.first  = false;
            validation.second = "The CameraSet should contain one and only one Camera.";
        }
    }

    return validation;
}

} // namespace sight::module::activity::validator::CameraSet
